/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/slave/cen-lnx64-xr/build/dom/interfaces/svg/nsIDOMSVGSwitchElement.idl
 */

#ifndef __gen_nsIDOMSVGSwitchElement_h__
#define __gen_nsIDOMSVGSwitchElement_h__


#ifndef __gen_nsIDOMSVGElement_h__
#include "nsIDOMSVGElement.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIDOMSVGSwitchElement */
#define NS_IDOMSVGSWITCHELEMENT_IID_STR "fb52b73c-3908-44c7-a390-2d40f8e8499c"

#define NS_IDOMSVGSWITCHELEMENT_IID \
  {0xfb52b73c, 0x3908, 0x44c7, \
    { 0xa3, 0x90, 0x2d, 0x40, 0xf8, 0xe8, 0x49, 0x9c }}

class NS_NO_VTABLE NS_SCRIPTABLE nsIDOMSVGSwitchElement : public nsIDOMSVGElement {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IDOMSVGSWITCHELEMENT_IID)

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIDOMSVGSwitchElement, NS_IDOMSVGSWITCHELEMENT_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIDOMSVGSWITCHELEMENT \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIDOMSVGSWITCHELEMENT(_to) \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIDOMSVGSWITCHELEMENT(_to) \
  /* no methods! */

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsDOMSVGSwitchElement : public nsIDOMSVGSwitchElement
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIDOMSVGSWITCHELEMENT

  nsDOMSVGSwitchElement();

private:
  ~nsDOMSVGSwitchElement();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsDOMSVGSwitchElement, nsIDOMSVGSwitchElement)

nsDOMSVGSwitchElement::nsDOMSVGSwitchElement()
{
  /* member initializers and constructor code */
}

nsDOMSVGSwitchElement::~nsDOMSVGSwitchElement()
{
  /* destructor code */
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIDOMSVGSwitchElement_h__ */
